/*******************************************************************************
**
**			DQF - Disk Quota File
**
********************************************************************************
*
*	purpose:	DQF (main) : example using dqf_exa_quota
*			DQF_exa_quota : examine disk quota entry
*			DQF_bal_quota : balance disk quota
*
*	pgm / date:	jrv, nasa/gsfc/634/sar  dec-1989
*			based heavily on diskquota.mar, chip tewskbury, jul-1989
*
*	mod:		jrv, 29-mar-1990
*			use new DQF struct in my dqfdef.h, remove .h from #inc,
*			modified naming convention of IOSB and ITM,
*			added IOSB status check, added $DASSGN call,
*			added uicdef.h usage
*
*	notes:		underlying VMS feature is $QIOW with ACP
*
*			see VAX/VMS I/O User's Reference Manual: Pt 1  (VMS 4.4)
*			Chapter 1 ACP-QIO Interface; 1.6.7.4 Disk Quotas
*
*			could additionally have DQF_add, _mod, and _rem
*
*			main program exits with '4d'x "channel interlock"
*			and i don't know why?!?
*
*******************************************************************************/

#include iodef			/* I/O function codes		*/
#include jpidef			/* Job/Process Information def	*/
#include ssdef			/* system service defintions	*/
#include fibdef			/* File Identification Block	*/
#include descrip		/* DESCRIPtor definitions	*/
#include uaidef			/* User Authorization Info	*/
#include uicdef			/* User Identification Code	*/
#include "dqfdef.h"		/* Disk Quota File defintions	*/

#define NULARG		0		/* longword 0 null argument	*/
#define MAX_ITMDSC_NUM	3		/* arbitraily large enough	*/
#define vms_error(status) (!(status & 1))

struct ITM				/* item list descriptor		*/
	{
	unsigned short	itm$w_buflen;	/* length in bytes of buffer	*/
	unsigned short	itm$w_itmcod;	/* symbolic item code		*/
	char	       *itm$a_buffer;	/* address of buffer		*/
	short	       *itm$a_retlen;	/* address of num bytes returned*/
 	};

struct IOSB				/* Input/Output Status Block	*/
	{
	unsigned short	iosb$w_status;	/* condition value status	*/
	unsigned short	iosb$w_count;	/* transfer count		*/
	unsigned long	iosb$l_info;	/* device specific information	*/
	};

typedef struct
{
  unsigned short status, terminator_offset, terminator, terminator_size;
} IOSB;

typedef struct
{
  short buffer_length;
  short item_code;
  long  buffer_address;
  long return_length_address;
} ITEMLIST;


char *vms_message();



DQF(pq, us, bal, over)
int *pq, *us, *bal, *over;

{
/*** diskquota definitions ***/

long int	DQF_exa_quota();	/* disk quota examine function	*/
long int	DQF_bal_quota();	/* disk quota balance function	*/
long int   	usage;	  	
long int	permquota;	
long int	overdraft;	
long int	balance;	
char		balance_type[10];	/* quota balance type OVER/AVAIL*/

/*** $GETJPI defintions ***/

char		usrnam[13];		/* user name string 12 char max	*/
$DESCRIPTOR	(usrnam_dsc, usrnam);	/* user name string descriptor	*/
short int	namlen;			/* user name return length	*/


/*** $GETUAI defintions ***/

struct ITM	itmlst[MAX_ITMDSC_NUM];	/* list of item descriptors	*/
long int	itmdsc_num;		/* item descriptor number	*/
long int	SYS$GETUAI();		/* GET User Authorization Info	*/
long int	status;		      	/* service condition value	*/
short int	retlen;			/* item descriptor return length*/

char		defdev[33];	     	/* default device		*/
$DESCRIPTOR	(defdev_dsc, defdev);	/* default device descriptor	*/
union uicdef	uic;			/* user identification code	*/


/*** prepare for the $GETUAI call ***/

    /* USERNAME: get the user name */

/*    GetMyUsername(usrnam); */

    status = LIB$GETJPI( &JPI$_USERNAME, NULARG, NULARG, NULARG,
			     &usrnam_dsc, &namlen );
    if (vms_error(status))
    {
	printf("lib$getjpi: %s", vms_message(status));
	exit(status);
    }
    usrnam[namlen] = '\0';
    usrnam_dsc.dsc$w_length = strlen( usrnam );		/* fix descriptor */

    itmdsc_num = 0;

    /* DEFDEV: default device - load item descriptor */

    itmlst[itmdsc_num].itm$w_buflen = 32;		/* 31 chars+ size-byte prefix */
    itmlst[itmdsc_num].itm$w_itmcod = UAI$_DEFDEV;
    itmlst[itmdsc_num].itm$a_buffer = defdev;
    itmlst[itmdsc_num].itm$a_retlen = &retlen;
    ++itmdsc_num;


    /* UIC: user identification code - load item descriptor */

    itmlst[itmdsc_num].itm$w_buflen = sizeof( union uicdef );
    itmlst[itmdsc_num].itm$w_itmcod = UAI$_UIC;
    itmlst[itmdsc_num].itm$a_buffer = &uic;
    itmlst[itmdsc_num].itm$a_retlen = &retlen;
    ++itmdsc_num;

    /* itemlist terminator */

    itmlst[itmdsc_num].itm$w_buflen = itmlst[itmdsc_num].itm$w_itmcod = 0;
    ++itmdsc_num;


/*** get the User Authorization Information ***/

    status = SYS$GETUAI(NULARG, NULARG,&usrnam_dsc, itmlst,
			NULARG, NULARG, NULARG );

    if (vms_error(status))
    {
	printf("sys$getuai: %s", vms_message(status));
	exit(status);
    }

/*    if( status != SS$_NORMAL ) LIB$SIGNAL( status );*/


    /* fix up the returned string */

    defdev[defdev[0]+1] = '\0';			/* null terminate the string */
    defdev_dsc.dsc$w_length = strlen( defdev ); /* fix descriptor length */
    defdev_dsc.dsc$a_pointer = defdev+1;	/* offset size-byte prefix   */


    /* examine quota entry */

     status = DQF_exa_quota( &uic, &defdev_dsc, &usage,
				    &permquota, &overdraft );
    if( status != SS$_NORMAL ) LIB$SIGNAL( status );


    /* balance quota */

    status = DQF_bal_quota( permquota, usage, &balance, balance_type );


    /* output the info */

    *pq = permquota;
    *us = usage;
    *bal = balance;
    *over = overdraft;

/*
    printf( "you have %d blocks used, %d %s,\n", usage, balance,
		balance_type );
    printf( "of %d authorized and permitted overdraft of %d blocks on %s\n",
		permquota, overdraft, defdev+1 );
*/
}	/* DQF */



/*** examine diskquota file entry (QIO-ACP) function ***/

long int DQF_exa_quota( puic, pdevnam_dsc, pusage, ppermquota, poverdraft )

/* parameter definitions */

union uicdef	*puic;			/* uic address			*/
struct dsc$descriptor_s	*pdevnam_dsc;	/* device name descriptor addr	*/
long int   	*pusage;	  	/* current usage address	*/
long int	*ppermquota;		/* permanent quota address	*/
long int	*poverdraft;		/* overdraft limit address	*/

{

/*** $ASSIGN / $DASSGN definitions ***/

long int	SYS$ASSIGN();		/* ASSIGN I/O channel service	*/
long int	SYS$DASSGN();		/* DASSGN I/O channel service	*/
unsigned short	chan;			/* channel number		*/


/*** $QIOW definitions ***/

long int	SYS$QIOW();             /* Queue Input Output and Wait	*/
static struct fibdef1 fib;		/* File Information Block (QUO)	*/
static struct dsc$descriptor fib_dsc;	/* FIB descriptor		*/

static struct DQF dqf;			/* Disk Quota File record	*/
static struct dsc$descriptor dqf_dsc;	/* DQF descriptor		*/

static struct IOSB iosb;		/* I/O status block		*/
long int	len;
long int	trap();			/* JL - error trap routine */ 

long int	status;			/* function return status	*/


/*** prepare for the $QIO call ***/

    /* load FIB and FIB descriptor */

    fib.fib$w_cntrlfunc = FIB$C_EXA_QUOTA;	/* examine disk quota	*/

    fib_dsc.dsc$w_length = FIB$K_LENGTH;	/* same as sizeof(fib)	*/
    fib_dsc.dsc$a_pointer = &fib;


    /* load DQF and DQF descriptor */

    dqf.dqf$l_uic = puic->uic$l_uic;

    dqf_dsc.dsc$w_length = DQF$K_LENGTH;	/* same as sizeof(dqf)	*/
    dqf_dsc.dsc$a_pointer = &dqf;


    /* assign an I/O channel to the device */

    status = SYS$ASSIGN( pdevnam_dsc, &chan, NULARG, NULARG );
    if (vms_error(status))
    {
	printf("sys$assign: %s", vms_message(status));
	exit(status);
    }

    /*if( status != SS$_NORMAL ) LIB$SIGNAL( status );*/


/*** issue an ACP-QIO to the device [chan, func, and p1-p4] ***/

    status = SYS$QIOW( NULARG,          /* event flag		*/
		   chan,		/* I/O channel number	*/
		   IO$_ACPCONTROL,	/* function code	*/
		   &iosb,		/* I/O status block	*/
		   NULARG, NULARG,	/* astadr, astprm	*/
		   &fib_dsc,		/* p1: FIB descriptor	*/
		   &dqf_dsc, 		/* p2: quota record input*/
		   &len,		/* p3: data length ret	*/
		   &dqf_dsc, 		/* p4: quota rec output */
		   NULARG, NULARG );	/* p5:, p6: unused	*/

    if (vms_error(status))
    {
	printf("sys$qiow: %s", vms_message(status));
	exit(status);
    }
/*    if( status != SS$_NORMAL) LIB$SIGNAL( status );
    if( iosb.iosb$w_status != SS$_NORMAL) LIB$SIGNAL( iosb.iosb$w_status );
*/

    /* deassign the I/O channel to the device */

    status = SYS$DASSGN( chan );
    if( status != SS$_NORMAL ) LIB$SIGNAL( status );


/*** stuff DQF output values into function parameters ***/

    *pusage     = dqf.dqf$l_usage;
    *ppermquota = dqf.dqf$l_permquota;
    *poverdraft = dqf.dqf$l_overdraft;


    return( status );

}	/* DQF_exa_quota */



/*** diskquota balance function ***/

long int DQF_bal_quota( permquota, usage, pbalance, balance_type )

long int   	usage;	  		/* current usage 		*/
long int	permquota;		/* permanent quota 		*/
long int	*pbalance;		/* quota balance (auth - used)	*/
char		balance_type[];		/* quota balance type OVER/AVAIL*/

{
    *pbalance = permquota - usage;

    if( *pbalance < 0 )
	  strcpy( balance_type, "OVERDRAWN" );
    else
	  strcpy( balance_type, "available" );

    return( SS$_NORMAL );

}	/* DQF_bal_quota */

GetMyUsername(char *username)
{
#define MAX_USERNAME_LEN 12
  int  status;
  char *pointer;
  IOSB iosb;
  ITEMLIST outlist[] =
  {
    { MAX_USERNAME_LEN+1, JPI$_USERNAME, username, 0 },
    { 0, 0, 0, 0 }
  };

  *username = '\0';
  status = sys$getjpiw(0, 0, 0, outlist, &iosb, 0, 0);
  if (vms_error(status))
    exit(status);
  else if (vms_error(iosb.status))
    exit(iosb.status);

  /* Trim the padded spaces */
  for (pointer=username; *pointer && *pointer!=' '; pointer++);
  *pointer = '\0';
}
                                                
