/*
 ***********************************************************************
 *
 *  (C) Copyright 1992-1993 The Trustees of Indiana University
 *
 *   Permission to use, copy, modify, and distribute this program for
 *   non-commercial use and without fee is hereby granted, provided that
 *   this copyright and permission notice appear on all copies and
 *   supporting documentation, the name of Indiana University not be used
 *   in advertising or publicity pertaining to distribution of the program
 *   without specific prior permission, and notice be given in supporting
 *   documentation that copying and distribution is by permission of
 *   Indiana University.
 *
 *   Indiana University makes no representations about the suitability of
 *   this software for any purpose. It is provided "as is" without express
 *   or implied warranty.
 *
 ***********************************************************************
 */

/*
**  Facility:  
**	Extract.c
**  Module description:
**      Extract folders from VMS Mail-11 converting those to Pine Folder format.
**  Authors:
**      Jacob Levanon, Larry Hughes - UCS/Services Development.
**
**  To-Do:
**	Make it generic with a qualifier to support many other folder formats (mhs, elm,
**	etc...)
*/

/*
**  Include files
*/
#include <stdio.h>
#include "mailmsgdef.h"
#include <climsgdef.h>
#include <string.h>
#include <descrip.h>
#include <time.h>
#include <ctype.h>
#include <ssdef.h>
#include <maildef.h>
#include <uaidef.h>
#include <stat.h>
#include <jpidef.h>
#include <nam.h>

/*
**  Prototypes
*/
char	*date_vms2unix();
void	display();
int	folder_routine();
void	get_time();
void	get_date();
int	mail_retrieve_message_headers();
int	mail_retrieve_message_text();
void    make_argv();
void    ProcessArgs();
void    patch_from_line();
void    perror();
void	show_syntax();
int	str_index();
char	*vms_message();

/*
**  Defines
*/
#define itemopen(ptr,itm) ptr = itm
#define itemadd(p,a,b,c,d) p->buffer_length = a; p->item_code = b; \
                           p->buffer_address = c; p->return_length_address = d; \
                           p++;
#define itemclose(ptr) itemadd(ptr,0,0,0,0);

#define vms_error(status) (!(status & 1))
#define MAX_ITEMS       10
#define Prologue	1
#define Epilogue	2
#define MIN_QUOTA	20

/*
**  Global declarations
*/
FILE    *outfolder;

typedef struct
{
  short buffer_length;
  short item_code;
  long  buffer_address;
  long return_length_address;
} ITEMLIST;

ITEMLIST nullist[] = { {0,0,0,0} };
ITEMLIST inlist[MAX_ITEMS];
ITEMLIST *inlist_ptr;
ITEMLIST outlist[MAX_ITEMS];
ITEMLIST *outlist_ptr;

typedef struct 
{
    unsigned short      msg_count;              
    unsigned int        user_context;           
    unsigned int        file_context;           
    unsigned int        message_context;        
    int                 newmail_selected;       
    char                mail_directory[256];    
    char                mail_file[256];         
} USER_MAIL;

struct node
{
    struct node *next;
    char *folder_name;
};

static	int	file_context=0, user_context=0, messages_selected=0;
	int	message_context=0;
	int	status = SS$_NORMAL;
	int	mode, length=0, user_length=0;
	int	records=1, number;
	int     AllFolders=FALSE, InfoFolders=FALSE, InfoQuota=FALSE;
	char	*prolog="extract:extract_prolog.txt", 
                *epilog="extract:extract_epilog.txt";

/*
**  Main function
*/
main (int argc, char *argv[])
{
    int num_tokens=0, i;
    struct node list = {0,0};
    struct node selected = {0,0};
    struct node *tmp;
    char fn[132], verb[9], sucrifice[132], *tokens[12];
    
    if (argc == 1)
    {
	show_syntax();
	exit(0);
    }

    ProcessArgs(&selected);

    status = MAIL$MAILFILE_BEGIN(&file_context, nullist, nullist);
    if (vms_error(status))
    {
	printf("mail$mailfile_begin: %s", vms_message(status));
	exit(status);
    }
    status=MAIL$MAILFILE_OPEN(&file_context, &nullist, &nullist);
    if (vms_error(status))
    {
	printf("mail$mailfile_open: %s", vms_message(status));
	exit(status);
    }

    itemopen(inlist_ptr,inlist);
    itemadd(inlist_ptr,4,MAIL$_MAILFILE_FOLDER_ROUTINE,folder_routine,0);
    itemadd(inlist_ptr,4,MAIL$_MAILFILE_USER_DATA,&list,0);
    itemadd(inlist_ptr,0,MAIL$_NOSIGNAL,0,0);
    itemclose(inlist_ptr);

    status=MAIL$MAILFILE_INFO_FILE(&file_context,inlist,nullist);
    if (vms_error(status))
    {
	printf("mail$mailfile_info_file: %s", vms_message(status));
	exit(status);
    }

    itemopen(inlist_ptr,inlist);
    itemadd(inlist_ptr,sizeof(file_context),
		    MAIL$_MESSAGE_FILE_CTX,&file_context,0);
    itemadd(inlist_ptr,0,MAIL$_NOSIGNAL,0,0);
    itemclose(inlist_ptr);

    status=MAIL$MESSAGE_BEGIN(&message_context,inlist, nullist);
    if (vms_error(status))
    {
	printf("mail$message_begin: %s", vms_message(status));
	exit(status);
    }

    if (AllFolders) tmp = &list;
    else tmp = &selected;

    if (tmp->next == NULL)
    {
	printf("No folders were found in your MAIL file...\n");
	exit(status);
    }

    while (tmp->next)
    {
	tmp = tmp->next;

	itemopen(inlist_ptr,inlist);
	itemadd(inlist_ptr,strlen(tmp->folder_name),
		MAIL$_MESSAGE_FOLDER,tmp->folder_name,0);
	itemadd(inlist_ptr,0,MAIL$_NOSIGNAL,0,0);
	itemclose(inlist_ptr);
	itemopen(outlist_ptr,outlist);
	itemadd(outlist_ptr,sizeof(messages_selected),MAIL$_MESSAGE_SELECTED,
	    &messages_selected,0);
	itemadd(outlist_ptr,0,MAIL$_NOSIGNAL,0,0);
	itemclose(outlist_ptr);

	status=MAIL$MESSAGE_SELECT(&message_context, inlist, outlist);
	if (vms_error(status))
	{
	    if (status == MAIL$_NOTEXIST)
		printf("Folder \"%s\" does not exist in your MAIL file...\n", tmp->folder_name);
	    else
		printf("mail$message_select: %s", vms_message(status));
	}

	if (messages_selected == 1)
	   sprintf(verb,"message");
	else
	   sprintf(verb,"messages");

	if (!InfoFolders & status == SS$_NORMAL)  /* actually do some work */
	{
	  check_quota(0);
          /* Check for a correct syntax for file names */

          strcpy(sucrifice, tmp->folder_name);
          make_argv(sucrifice, &num_tokens, tokens, ".");
          if (num_tokens < 2)
            sprintf(fn,"%s",tmp->folder_name);
	  else if (num_tokens == 2)	
            sprintf(fn,"%s.%s",tokens[0],tokens[1]);
	  else if (num_tokens > 2)	/* case of illegal filename - fix it */
            sprintf(fn,"%s.%s_%s",tokens[0],tokens[1],tokens[2]);

	  if ((outfolder = fopen(fn, "w")) == NULL)
	  {
	    printf("Unable to open file %s\n", fn);
	    perror("open");
	    exit(1);
	  }
	  printf("Folder %s contains %d %s - extracting...\n",
		tmp->folder_name,messages_selected, verb);
	  for (i=1; i<=messages_selected; i++)
	  {
	    mail_retrieve_message_headers(i,message_context);
	    mail_retrieve_message_text(message_context);
	    status = fprintf(outfolder, "\n");
	    if (status < 0)
	    {
		perror("writing folder end");
		exit();
	    }
	  }
	  fclose(outfolder);
	}

	else if (status == SS$_NORMAL)		/* just display some info */
	  printf("Folder %s - contains %d %s...\n", 
			tmp->folder_name,messages_selected,verb);
    }

    if (InfoFolders)
	check_quota(1);

    if (!InfoFolders & status == SS$_NORMAL) 
	display(epilog);
    exit(1);
}

/*
**
*/
check_quota(int flag)
{
  long int usage;	  	/* current usage 		*/
  long int permquota;		/* permanent quota 		*/
  long int overdraft;		/* overdraft limit 		*/
  long int balance;		/* quota balance (auth - used)	*/
       float   availquota;

  DQF(&permquota, &usage, &balance, &overdraft);
  if (flag)
  {
    printf("\nQuota:   You have %d blocks used, %d available,\n", usage, balance);
    printf("\t of %d authorized and permitted overdraft of %d blocks\n",
	    permquota, overdraft);
  }
  else 
  {
    if (usage != 0.)
    {
     availquota = (balance+overdraft)*100/usage;
     if (availquota < MIN_QUOTA)
     {
      printf("Warning: %5.1f%% of your disk quota is occupied.  The available\n", 100-availquota);
      printf("\t quota might be too low to extract all of your folders.\n");
      printf("\t You have %d blocks used, %d available,\n", usage, balance);
      printf("\t of %d authorized and permitted overdraft of %d blocks\n",
	    permquota, overdraft);
     }
    }
  }
}

/*
** Change VMS date format to Unix
*/
char *date_vms2unix(char *vmsdate)
{
  int  year, month, day, saveyear, dayofweek;
  long julian, j1, j2, j3;
  char *daynames[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
  char time[9];
  char monthname[4];
  static char buffer[32];

  memcpy(buffer,vmsdate+7,4); buffer[4] = '\0';
  saveyear = atoi(buffer);
  year = saveyear;

  memcpy(monthname,vmsdate+3,3); monthname[3] = '\0';
  switch (monthname[0])
  {
    case 'J':
      if (monthname[1] == 'A')
        month = 1;
      else if (monthname[2] == 'N')
        month = 6;
      else
        month = 7;
      break;
    case 'F':
      month = 2;
      break;
    case 'M':
      if (monthname[2] == 'R') month = 3; else month = 5;
      break;
    case 'A':
      if (monthname[1] == 'P') month = 4; else month = 8;
      break;
    case 'S':
      month = 9;
      break;
    case 'O':
      month = 10;
      break;
    case 'N':
      month = 11;
      break;
    default:
      month = 12;
      break;
  }
  monthname[1] = tolower(monthname[1]);
  monthname[2] = tolower(monthname[2]);

  memcpy(buffer,vmsdate,2); buffer[2] = '\0';
  day = atoi(buffer);

  memcpy(time,vmsdate+12,8); time[8] = '\0';

  if (year < 1584)
    buffer[0] = '\0';
  else
  {
    if (month > 2)
      month -= 3;
    else
    {
      month += 9;
      year--;
    }
    j1 = 146097 * (year / 100) / 4;
    j2 = 1461L  * (year % 100) / 4;
    j3 = (153L  * month + 2) / 5 + day + 1721119;
    julian = j1 + j2 + j3;
    dayofweek = (int)(++julian % 7);
    sprintf(buffer,"%-3s %-3s %02d %-8s %02d", daynames[dayofweek], 
            monthname, day, time, saveyear);
  }
  return(buffer);
}
                                       
/*
** Display contents of a file
*/
void display(char *fn)
{
    FILE *infile;
    char line[81];    
    int status;

    if ((infile= fopen(fn, "r")) == NULL)
    {
	printf("Unable to open file %s\n", fn);
	exit(1);
    }
    while ((status = fgets(line, 80, infile)) != NULL)
	printf("%s", line);
}

/*
** Folder_routine generates a list of all the VMS Mail-11 folders
*/
int folder_routine(struct node *list, struct dsc$descriptor *name)
{
    if (name->dsc$w_length)
    {
	while (list->next)
	  list = list->next;

	list->next = malloc(sizeof(struct node));
	list = list->next;
	list->next = 0;
	list->folder_name = malloc(name->dsc$w_length+1);
	strncpy(list->folder_name, name->dsc$a_pointer,name->dsc$w_length); 
 	list->folder_name[name->dsc$w_length] = '\0'; 
    }
    return(SS$_NORMAL);
}

/* 
** Get Date 
*/
void get_date(char *date_str)
{
  time_t calendar_time;
  struct tm *local_time;

  calendar_time = time(0);
  local_time = localtime(&calendar_time);
  ++local_time->tm_mon;
  sprintf(date_str,"19%02d-%02d-%02d",
          local_time->tm_year,local_time->tm_mon,local_time->tm_mday);
}
                  
/*
** Get_time
*/
void get_time(char *time_str)
{
  time_t calendar_time;
  struct tm *local_time;

  calendar_time = time(0);
  local_time = localtime(&calendar_time);
  sprintf(time_str,"%02d:%02d:%02d",
          local_time->tm_hour,local_time->tm_min,local_time->tm_sec);
}
          
/*
** Retrieve all the HEADERS for the current message
*/
int mail_retrieve_message_headers(int message_id, int message_context)
{
#define MAXHEADERS      4
#define FROM_LINE       1
#define TO_LINE         2
#define CC_LINE         3
#define SUBJECT_LINE    4

  int  length;
  int  status;
  int  headers;
  short int record_type;
  char *pointer;
  char message_date[32], date[32];
  char buffer[256], from[256], xfrom[256], cc[256], to[256], subject[256];
  char date_str[11];
  char time_str[9];

  get_time(time_str);
  get_date(date_str);

  /* Get message date */
  itemopen(inlist_ptr, inlist);
  itemadd(inlist_ptr, sizeof(message_id), MAIL$_MESSAGE_ID,
          &message_id, 0);
  itemadd(inlist_ptr, 0, MAIL$_NOSIGNAL, 0, 0);
  itemclose(inlist_ptr);
  itemopen(outlist_ptr, outlist);
  itemadd(outlist_ptr, sizeof(message_date), MAIL$_MESSAGE_DATE,
              message_date, &length);
  itemadd(outlist_ptr, 0, MAIL$_NOSIGNAL, 0, 0);
  itemclose(outlist_ptr);
  status = mail$message_get(&message_context,inlist,outlist);
  if (vms_error(status) & (status != MAIL$_NOMOREMSG))
  { 
    printf("mail_retrieve_message_headers:mail$message_get:DATE: \n\t%s\n", 
	    vms_message(status));
    return(0);
  }
  message_date[length] = '\0';
  strcpy(date, date_vms2unix(message_date));

  /* Prepare for remaining headers */
  itemopen(inlist_ptr,inlist);
  itemadd(inlist_ptr,0,MAIL$_MESSAGE_CONTINUE,0,0);
  itemadd(inlist_ptr,0,MAIL$_NOSIGNAL,0,0);
  itemclose(inlist_ptr);
  itemopen(outlist_ptr,outlist);
  itemadd(outlist_ptr,sizeof(buffer)-1,MAIL$_MESSAGE_RECORD,buffer,
          &length);
  itemadd(outlist_ptr,sizeof(record_type),MAIL$_MESSAGE_RECORD_TYPE,
          &record_type,0);
  itemadd(outlist_ptr,0,MAIL$_NOSIGNAL,0,0);
  itemclose(outlist_ptr);

  /* Process remaining headers */
  status = SS$_NORMAL;
  for (headers=1; headers<=MAXHEADERS && !vms_error(status); headers++)
  {
    memset(buffer, '\0', sizeof(buffer));
    status = mail$message_get(&message_context,inlist,outlist);
    if (vms_error(status) & (status != MAIL$_NOMOREREC))
    {
	printf("mail_retrieve_message_headers:mail$message_get:HEADERS\n\t%s\n", 
		vms_message(status));
	return(0);
    }
    else
    {
      while ((pointer = strchr(buffer,'\t')) != NULL) *pointer = ' ';
      switch (headers)
      {
        case FROM_LINE:
          strcpy(from, &buffer[6]);
          strcpy(xfrom, from);
          patch_from_line(from);
          break;

        case CC_LINE:
          strcpy(cc, &buffer[4]);
          lower(cc);
          break;

        case SUBJECT_LINE:
          strcpy(subject, &buffer[6]);
          break;

        case TO_LINE:
          strcpy(to, &buffer[4]);
          lower(to);
          break;
      }
    }
  }

  /* Print headers */
  status = fprintf(outfolder, "From %s %s\n", from, date);
  if (status < 0)
  {
    perror("writing from");
    exit();
  }
  status = (outfolder, "Date: %s\n", date);
  if (status < 0)
  {
    perror("writing date");
    exit();
  }
  status = fprintf(outfolder, "From: %s\n", from);
  if (status < 0)
  {
    perror("writing from");
    exit();
  }
  status = fprintf(outfolder, "Subject: %s\n", subject);
  if (status < 0)
  {
    perror("writing subject");
    exit();
  }
  status = fprintf(outfolder, "To: %s\n", to);
  if (status < 0)
  {
    perror("writing to");
    exit();
  }
  status = fprintf(outfolder, "Cc: %s\n", cc);
  if (status < 0)
  {
    perror("writing Cc");
    exit();
  }
  status = fprintf(outfolder, "X-VMS-From: %s\n", xfrom);
  if (status < 0)
  {
    perror("writing X-VMS-From");
    exit();
  }
  return(1);
}

/*
** Retrieve the body of the current message
*/
int mail_retrieve_message_text(int message_context)
{
  static int zero = 0;
  static int one  = 1;
  int  bytes;
  int  status;
  int  length;
  int  save_errno;
  int  retval  = FALSE;
  int  sending = TRUE;
  char buffer[256];

  itemopen(inlist_ptr, inlist);
  itemadd(inlist_ptr, 0, MAIL$_MESSAGE_CONTINUE, 0, 0);
  itemadd(inlist_ptr, 0, MAIL$_NOSIGNAL, 0, 0);
  itemclose(inlist_ptr);

  itemopen(outlist_ptr, outlist);
  itemadd(outlist_ptr, sizeof(buffer)-2, MAIL$_MESSAGE_RECORD,
          &buffer, &length);
  itemadd(outlist_ptr, 0, MAIL$_NOSIGNAL, 0, 0);
  itemclose(outlist_ptr);

  while (1)
  {
      status = mail$message_get(&message_context, inlist, outlist);
      if (vms_error(status) & (status != MAIL$_NOMOREREC))
      {
        printf("mail$message_get: %s\n", vms_message(status));
	return(0);
      }
      if (status == MAIL$_NOMOREREC)
	break;
      buffer[length] = '\0';
      status = fprintf(outfolder, "%s\n", buffer);
      if (status < 0)
      {
	perror("writing message body");
	exit();
      }
  }
  return(1);
}


/*
** Make Argv 
*/
void make_argv(char *string, int *argc, char *argv[], char *delimiters)
{
#define isdelimiter(a,b) (strchr(b,a) != NULL)
  register char *cpointer;
  register char **argpointer = argv;

  *argc = 0;
  cpointer = string;
  while (*cpointer)
  {
    while (*cpointer && isdelimiter(*cpointer,delimiters)) cpointer++;
    if (*cpointer == '\0') break;
    if (*cpointer == '\"')
    {
      cpointer++;
      *argpointer++ = cpointer;
      (*argc)++;
      while (*cpointer && (*cpointer != '\"')) cpointer++;
    }
    else
    {
      *argpointer++ = cpointer;
      (*argc)++;
      while (*cpointer && !isdelimiter(*cpointer,delimiters)) cpointer++;
    }
    if (*cpointer == '\0') break;
    *(cpointer++) = '\0';
  }
  *(argpointer++) = 0;
}

/*
** Patch From Line
*/
void patch_from_line(char *from_line)
{
  int  num_tokens;
  char input[256];
  char output[256];
  char *myhostname="ezmail";
  char *tokens[128];  /* make it huge or the stack might get corrupted */
  char *pointer;

  /* Token #1 is address, #2 (if it exists) is personal name */
  strcpy(input, from_line);
  make_argv(input, &num_tokens, tokens, " ");
  if ((num_tokens < 1) || (num_tokens > 2))
  {
    printf("Parsing ERROR:\n");
    printf("%s\n", from_line);
    printf("The address above contains an unbalanced number of quotes, and thus\n");
    printf("cannot be automatically converted to an Internet-style address.\n");
    printf("You will have to convert it yourself before replying to this message.\n");
    return;
  }

  /* Lower case the address (but not the personal name) */
  lower(tokens[0]);

  /*
   * If address is of the form:
   *
   *    FOO%"someone@somewhere"    -or-
   *    NODE::"someone@somewhere"
   *
   * Just use what is in the quotes.
   */
  if (pointer = (char *)strchr(tokens[0], '"'))
  {
    strcpy(output, pointer+1);
    pointer = output + strlen(output) - 1;
    if (*pointer == '"') *pointer = '\0';
  }

  /*
   * If address is of the form:
   *
   *   NODE::USER
   *
   * Convert it to the form user@node
   */
  else if (pointer = (char *)strstr(tokens[0], "::"))
  {
    strcpy(output, pointer+2);
    strcat(output, "@");
    *pointer = '\0';
    strcat(output, tokens[0]);
  }

  /*
   * None of the above; it's local, so append "@hostname"
   */
  else
  {
    sprintf(output, "%s@%s", tokens[0], myhostname);
  }

  /*
   * Put the address in angle brackets, and prepend personal name if it exists.
   */
  from_line[0] = '\0';
#ifdef PERSONAL_NAME
  if (tokens[1]) sprintf(from_line, "\"%s\" ", tokens[1]);
#endif PERSONAL_NAME
  if (*output == '<')
    sprintf(from_line+strlen(from_line), "%s", output);
  else
    sprintf(from_line+strlen(from_line), "<%s>", output);
}

/************************************************************************
 * Function:
 *     ProcessArgs
 *
 * Description:
 *     Process command arguments.
 *
 * Arguments:
 *     None.

 * Returns:
 *     Nothing.
 ************************************************************************/
void ProcessArgs(struct node *list)
{
  int status;
  int order = 1;
  unsigned short cliLength;
  char value[256];
  $DESCRIPTOR(valueDesc, value);
  char *tmp;

  /* these must match the parameter labels in the .CLD file */
  $DESCRIPTOR(allDesc,      "ALL");
  $DESCRIPTOR(helpDesc,      "HELP");
  $DESCRIPTOR(foldersDesc,     "FOLDERS");
  $DESCRIPTOR(infoDesc,      "INFO");

  /* /HELP */
  status = cli$present(&helpDesc);
  switch (status)
  {
    case CLI$_ABSENT:
      break;
    case CLI$_PRESENT:
      show_syntax();
      exit(status);
      break;
    default:
      exit(status);
  }

  /* /FOLDERS*/
  status = cli$present(&foldersDesc);
  switch (status)
  {
    case CLI$_ABSENT:
      AllFolders=TRUE;
      break;
    case CLI$_PRESENT:
      do
      {
	status = cli$get_value(&foldersDesc,  &valueDesc, &cliLength);
	switch (status)
	{
	  case CLI$_ABSENT:
	    break;
	  case SS$_NORMAL:
          case CLI$_COMMA:
	    value[cliLength] = '\0';
	    list->next = malloc(sizeof(struct node));
	    list = list->next;
	    list->next = 0;
	    list->folder_name = malloc(cliLength+1);
	    strcpy(list->folder_name, value); 
	    break;
	  default:
	    exit(status);
	}
      }	while (status == CLI$_COMMA);
      break;
    case CLI$_NEGATED:
      break;
    default:
      break;
  }

  /* /ALL */
  status = cli$present(&allDesc);
  switch (status)
  {
    case CLI$_ABSENT:
      break;
    case CLI$_PRESENT:
      AllFolders=TRUE;
      break;
    default:
      exit(status);
  }

  /* /INFO */
  status = cli$present(&infoDesc);
  switch (status)
  {
    case CLI$_ABSENT:
      break;
    case CLI$_PRESENT:
      InfoFolders=TRUE;
      break;
    default:
      exit(status);
  }

}

/*
** VMS Message String
*/
char *vms_message(unsigned int code)
{
  static   char  message[256];
  unsigned short length = 0;
  struct   dsc$descriptor message_dx;

  memset(message,0,sizeof(message));
  message_dx.dsc$w_length  = sizeof(message);
  message_dx.dsc$a_pointer = &message;
  sys$getmsg(code,&length,&message_dx,0xf,0);
  message[length] = '\0';
  return(&message);
}

/*
**  Show_syntax
*/
void show_syntax()
{
    display(prolog);
    printf("\n");
    printf("      Syntax: extract/Qualifier=[values]\n\n");
 /*   printf("        extract/Qualifier=[values]\n\n");*/
    printf("      where qualifiers are:\n");
    printf("        /HELP\n\t\t- show this help.\n");
    printf("        /INFO\n\t\t- show information on all folders in the mail file.\n");
    printf("        /ALL\n\t\t- extract all existing mail folders.\n");
    printf("        /FOLDERS=(folder[,folder1...])\n\t\t- extract only listed folders.\n");
}

/*
** Str_index	
*/	
int str_index(register char s[], register char t[])
{						
  register int i,k,j;				
						
  for (i=0; s[i] != '\0'; i++)			
  {						
    for (j=i, k=0; t[k] != '\0' && s[j] == t[k]; j++, k++);
    if (t[k] == '\0')					
      return(i);					
  }		
  return (-1);	
}
